@@ -90,6 +90,7 @@ void MMCifReader::ClearState()
   revisions_.clear();
   revision_types_.clear();
   database_PDB_rev_added_ = false;
+  entity_branch_link_map_.clear();
 }
 
 void MMCifReader::SetRestrictChains(const String& restrict_chains)
@@ -362,7 +363,30 @@ bool MMCifReader::OnBeginLoop(const StarLoopDesc& header)
     indices_[PDS_RECVD_INITIAL_DEPOSITION_DATE]
      = header.GetIndex("recvd_initial_deposition_date");
     cat_available = true;
-  }
+  } else if (header.GetCategory() == "pdbx_entity_branch") {
+    category_ = PDBX_ENTITY_BRANCH;
+    // mandatory
+    this->TryStoreIdx(BR_ENTITY_ID, "entity_id", header);
+    this->TryStoreIdx(BR_ENTITY_TYPE, "type", header); 
+    cat_available = true;
+ } else if (header.GetCategory() == "pdbx_entity_branch_link") {
+    category_ = PDBX_ENTITY_BRANCH_LINK;
+    // mandatory
+    this->TryStoreIdx(BL_ENTITY_ID, "entity_id", header);
+    this->TryStoreIdx(BL_ATOM_ID_1, "atom_id_1", header);
+    this->TryStoreIdx(BL_ATOM_ID_2, "atom_id_2", header);
+    this->TryStoreIdx(BL_COMP_ID_1, "comp_id_1", header);
+    this->TryStoreIdx(BL_COMP_ID_2, "comp_id_2", header);
+    this->TryStoreIdx(BL_ENTITY_BRANCH_LIST_NUM_1, "entity_branch_list_num_1",
+                      header);
+    this->TryStoreIdx(BL_ENTITY_BRANCH_LIST_NUM_2, "entity_branch_list_num_2",
+                      header);
+    // optional items
+    indices_[BL_ATOM_STEREO_CONFIG_1] = header.GetIndex("atom_stereo_config_1");
+    indices_[BL_ATOM_STEREO_CONFIG_2] = header.GetIndex("atom_stereo_config_2");
+    indices_[BL_VALUE_ORDER] = header.GetIndex("value_order");
+    cat_available = true;
+ }
   category_counts_[category_]++;
   return cat_available;
 }
@@ -636,47 +660,50 @@ void MMCifReader::ParseAndAddAtom(const std::vector<StringRef>& columns)
                 columns[indices_[GROUP_PDB]][0]=='H');
 }
 
+MMCifReader::MMCifEntityDescMap::iterator MMCifReader::GetEntityDescMapIterator(
+  const String& entity_id)
+{
+  MMCifEntityDescMap::iterator edm_it = entity_desc_map_.find(entity_id);
+  // if the entity ID is not already stored, insert it with empty values
+  if (edm_it == entity_desc_map_.end()) {
+    MMCifEntityDesc desc = {.type=mol::CHAINTYPE_N_CHAINTYPES,
+                            .details="",
+                            .seqres=""};
+    edm_it = entity_desc_map_.insert(entity_desc_map_.begin(),
+                                     MMCifEntityDescMap::value_type(entity_id,
+                                                                    desc));
+  }
+  return edm_it;
+}
+
 void MMCifReader::ParseEntity(const std::vector<StringRef>& columns)
 {
-  bool store = false; // is it worth storing this record?
-  MMCifEntityDesc desc;
+  MMCifEntityDescMap::iterator edm_it =
+    GetEntityDescMapIterator(columns[indices_[E_ID]].str());
 
   // type
   if (indices_[E_TYPE] != -1) {
-    desc.type = mol::ChainTypeFromString(columns[indices_[E_TYPE]]);
-    store = true;
+    // only use the entity type if no other is set, entity_poly type is
+    // more precise, so if that was set before just leave it in
+    if (edm_it->second.type == mol::CHAINTYPE_N_CHAINTYPES) {
+      edm_it->second.type = mol::ChainTypeFromString(columns[indices_[E_TYPE]]);
+    }
+  } else {
+    // don't deal with entities without type
+    entity_desc_map_.erase(edm_it);
+    return;
   }
 
   // description
   if (indices_[PDBX_DESCRIPTION] != -1) {
-    desc.details = columns[indices_[PDBX_DESCRIPTION]].str();
-  } else {
-    desc.details = "";
-  }
-
-  if (store) {
-    desc.seqres = "";
-    entity_desc_map_.insert(
-                   MMCifEntityDescMap::value_type(columns[indices_[E_ID]].str(),
-                                                  desc)
-                            );
+    edm_it->second.details = columns[indices_[PDBX_DESCRIPTION]].str();
   }
 }
 
 void MMCifReader::ParseEntityPoly(const std::vector<StringRef>& columns)
 {
-  // we assume that the entity cat. ALWAYS comes before the entity_poly cat.
-  // search entity
   MMCifEntityDescMap::iterator edm_it =
-    entity_desc_map_.find(columns[indices_[ENTITY_ID]].str());
-
-  if (edm_it == entity_desc_map_.end()) {
-    throw IOException(this->FormatDiagnostic(STAR_DIAG_ERROR,
-                     "'entity_poly' category defined before 'entity' for id '" +
-                                            columns[indices_[ENTITY_ID]].str() +
-                                             "' or missing.",
-                                             this->GetCurrentLinenum()));
-  }
+    GetEntityDescMapIterator(columns[indices_[ENTITY_ID]].str());
 
   // store type
   if (indices_[EP_TYPE] != -1) {
@@ -1541,6 +1568,14 @@ void MMCifReader::OnDataRow(const StarLoopDesc& header,
     LOG_TRACE("processing pdbx_database_status entry");
     this->ParsePdbxDatabaseStatus(columns);
     break;
+  case PDBX_ENTITY_BRANCH:
+    LOG_TRACE("processing pdbx_entity_branch entry");
+    this->ParsePdbxEntityBranch(columns);
+    break;
+  case PDBX_ENTITY_BRANCH_LINK:
+    LOG_TRACE("processing pdbx_entity_branch_link entry");
+    this->ParsePdbxEntityBranchLink(columns);
+    break;
   default:
     throw IOException(this->FormatDiagnostic(STAR_DIAG_ERROR,
                        "Uncatched category '"+ header.GetCategory() +"' found.",
@@ -1679,6 +1714,68 @@ void MMCifReader::ParseStructRefSeqDif(const std::vector<StringRef>& columns)
  }
 }
 
+void MMCifReader::ParsePdbxEntityBranch(const std::vector<StringRef>& columns)
+{
+  // get entity/ descreption entry
+  MMCifEntityDescMap::iterator edm_it =
+    GetEntityDescMapIterator(columns[indices_[BR_ENTITY_ID]].str());
+
+  // store type
+  if (indices_[BR_ENTITY_TYPE] != -1) {
+    edm_it->second.type = mol::ChainTypeFromString(columns[indices_[EP_TYPE]]);
+  }
+}
+
+void MMCifReader::ParsePdbxEntityBranchLink(const std::vector<StringRef>& columns)
+{
+  MMCifPdbxEntityBranchLink link_pair;
+
+  String entity_id(columns[indices_[BL_ENTITY_ID]].str());
+
+  // list of entities -> pairs of info for link
+  link_pair.res_num_1 =
+    this->TryGetInt(columns[indices_[BL_ENTITY_BRANCH_LIST_NUM_1]],
+                    "pdbx_entity_branch_link.entity_branch_list_num_1");
+  link_pair.cmp_1 = columns[indices_[BL_COMP_ID_1]].str();
+  link_pair.atm_nm_1 = columns[indices_[BL_ATOM_ID_1]].str();
+  link_pair.res_num_2 =
+    this->TryGetInt(columns[indices_[BL_ENTITY_BRANCH_LIST_NUM_2]],
+                    "pdbx_entity_branch_link.entity_branch_list_num_2");
+  link_pair.cmp_2 = columns[indices_[BL_COMP_ID_2]].str();
+  link_pair.atm_nm_2 = columns[indices_[BL_ATOM_ID_2]].str();
+
+  /*if (indices_[BL_ATOM_STEREO_CONFIG_1] != -1) {
+    char A = *columns[indices_[BL_ATOM_STEREO_CONFIG_1]].begin();
+  }*/
+  // check stereo values to be N S R
+  /*if (indices_[BL_ATOM_STEREO_CONFIG_2] != -1) {
+  }*/
+  // check value order
+  if (indices_[BL_VALUE_ORDER] != -1) {
+    link_pair.bond_order = MMCifValueOrderToOSTBondOrder(
+                                              columns[indices_[BL_VALUE_ORDER]]);
+  } else {
+    link_pair.bond_order = 1;
+  }
+
+  std::pair<MMCifPdbxEntityBranchLinkMap::iterator, bool> rit;
+
+  // check if element already exists
+  MMCifPdbxEntityBranchLinkMap::iterator blm_it =
+    entity_branch_link_map_.find(entity_id);
+
+  // if the entity was not seen before, create it in the map
+  if (blm_it == entity_branch_link_map_.end()) {
+    rit = entity_branch_link_map_.insert(
+                   MMCifPdbxEntityBranchLinkMap::value_type(entity_id,
+                                      std::vector<MMCifPdbxEntityBranchLink>()));
+    blm_it = rit.first;
+  }
+
+  // add the link pair
+  blm_it->second.push_back(link_pair);
+}
+
 void MMCifReader::OnEndData()
 {
   mol::XCSEditor editor=ent_handle_.EditXCS(mol::BUFFERED_EDIT);
@@ -1686,10 +1783,12 @@ void MMCifReader::OnEndData()
   // process chain types
   std::vector<std::pair<mol::ChainHandle, String> >::const_iterator css;
   MMCifEntityDescMap::const_iterator edm_it;
+  MMCifPdbxEntityBranchLinkMap::const_iterator blm_it;
+  std::vector<MMCifPdbxEntityBranchLink>::const_iterator bl_it;
   String pdb_auth_chain_name;
   for (css = chain_id_pairs_.begin(); css != chain_id_pairs_.end(); ++css) {
+    // chain description
     edm_it = entity_desc_map_.find(css->second);
-
     if (edm_it != entity_desc_map_.end()) {
       editor.SetChainType(css->first, edm_it->second.type);
       editor.SetChainDescription(css->first, edm_it->second.details);
@@ -1713,6 +1812,22 @@ void MMCifReader::OnEndData()
       LOG_WARNING("No entity description found for atom_site.label_entity_id '"
                   << css->second << "'");
     }
+    // find
+    blm_it = entity_branch_link_map_.find(css->second);
+    // store linker pair
+    if (blm_it != entity_branch_link_map_.end()) {
+      for (bl_it = blm_it->second.begin(); bl_it != blm_it->second.end();
+           ++bl_it) {
+        mol::ResidueHandle res1 = css->first.FindResidue(to_res_num(
+                                                         bl_it->res_num_1, ' '));
+        mol::ResidueHandle res2 = css->first.FindResidue(to_res_num(
+                                                         bl_it->res_num_2, ' '));
+        info_.AddEntityBranchLink(css->first.GetName(),
+                                  res1.FindAtom(bl_it->atm_nm_1),
+                                  res2.FindAtom(bl_it->atm_nm_2),
+                                  bl_it->bond_order);
+      }
+    }
   }
 
   // process citations (couple with authors
@@ -1807,4 +1922,64 @@ void MMCifReader::OnEndData()
            << strand_list_.size() << " strands");
 }
 
+unsigned char MMCifValueOrderToOSTBondOrder(const StringRef value_order)
+{
+  if (value_order == StringRef("sing", 4)) {
+      return 1;
+  }
+  if (value_order == StringRef("doub", 4)) {
+      return 2;
+  }
+  if (value_order == StringRef("trip", 4)) {
+      return 3;
+  }
+  LOG_WARNING("Non-covered bond order found: '" << value_order << "'");
+  if (value_order == StringRef("arom", 4)) {
+      return 4;
+  }
+  if (value_order == StringRef("delo", 4)) {
+      return 5;
+  }
+  if (value_order == StringRef("pi", 2)) {
+      return 6;
+  }
+  if (value_order == StringRef("poly", 4)) {
+      return 7;
+  }
+  if (value_order == StringRef("quad", 4)) {
+      return 8;
+  }
+  return 1;
+}
+
+String OSTBondOrderToMMCifValueOrder(const unsigned char bond_order)
+{
+  if (bond_order == 1) {
+    return String("sing");
+  }
+  if (bond_order == 2) {
+    return String("doub");
+  }
+  if (bond_order == 3) {
+    return String("trip");
+  }
+  if (bond_order == 4) {
+    return String("arom");
+  }
+  if (bond_order == 5) {
+      return String("delo");
+  }
+  if (bond_order == 6) {
+      return String("pi");
+  }
+  if (bond_order == 7) {
+      return String("poly");
+  }
+  if (bond_order == 8) {
+      return String("quad");
+  }
+  LOG_WARNING("Unknow bond order found: '" << (int)bond_order << "'");
+  return String("");
+}
+
 }}
