@@ -61,6 +61,8 @@ public:
   using MMCifReader::ParseStruct;
   using MMCifReader::ParseStructConf;
   using MMCifReader::ParseStructSheetRange;
+  using MMCifReader::ParsePdbxEntityBranch;
+  using MMCifReader::ParsePdbxEntityBranchLink;
   using MMCifReader::TryStoreIdx;
   using MMCifReader::SetRestrictChains;
   using MMCifReader::SetReadSeqRes;
@@ -310,9 +312,13 @@ BOOST_AUTO_TEST_CASE(mmcif_unknown_entity_type)
   columns.push_back(StringRef("polymer", 7));
   BOOST_CHECK_NO_THROW(tmmcif_p.ParseEntity(columns));
   columns.pop_back();
+  columns.pop_back();
+  columns.push_back(StringRef("2", 1));
   columns.push_back(StringRef("non-polymer", 11));
   BOOST_CHECK_NO_THROW(tmmcif_p.ParseEntity(columns));
   columns.pop_back();
+  columns.pop_back();
+  columns.push_back(StringRef("3", 1));  
   columns.push_back(StringRef("water", 5));
   BOOST_CHECK_NO_THROW(tmmcif_p.ParseEntity(columns));
   BOOST_TEST_MESSAGE("          done.");
@@ -320,6 +326,8 @@ BOOST_AUTO_TEST_CASE(mmcif_unknown_entity_type)
   // negative
   BOOST_TEST_MESSAGE("          unknown type...");
   columns.pop_back();
+  columns.pop_back();
+  columns.push_back(StringRef("4", 1));
   columns.push_back(StringRef("foo", 3));
   BOOST_CHECK_THROW(tmmcif_p.ParseEntity(columns), Error);
   BOOST_TEST_MESSAGE("          done.");
@@ -403,20 +411,6 @@ BOOST_AUTO_TEST_CASE(mmcif_entity_poly_tests)
   seq::SequenceHandle curr = seqres.FindSequence("A");
   BOOST_CHECK(curr.GetString() == "VTI");
 
-  BOOST_TEST_MESSAGE("          testing missing corresponding entity entry...");
-  {
-    mol::EntityHandle eh = mol::CreateEntity();
-    std::vector<StringRef> columns;
-    TestMMCifReaderProtected tmmcif_p("testfiles/mmcif/atom_site.mmcif", eh);
-
-    tmmcif_h.SetCategory(StringRef("entity_poly", 11));
-    tmmcif_h.Add(StringRef("entity_id", 9));
-    tmmcif_p.OnBeginLoop(tmmcif_h);
-
-    columns.push_back(StringRef("1", 1));
-    BOOST_CHECK_THROW(tmmcif_p.ParseEntityPoly(columns), IOException);
-  }
-  BOOST_TEST_MESSAGE("          done.");
   BOOST_TEST_MESSAGE("          testing type recognition...");
   {
     TestMMCifReaderProtected tmmcif_p("testfiles/mmcif/atom_site.mmcif", eh);
@@ -467,9 +461,8 @@ columns.push_back(StringRef("polydeoxyribonucleotide/polyribonucleotide hybrid",
     columns.push_back(StringRef("other", 5));
     BOOST_CHECK_NO_THROW(tmmcif_p.ParseEntityPoly(columns));
     columns.pop_back();
-    columns.pop_back();
     columns.push_back(StringRef("badbadprion", 11));
-    BOOST_CHECK_THROW(tmmcif_p.ParseEntityPoly(columns), IOException);
+    BOOST_CHECK_THROW(tmmcif_p.ParseEntityPoly(columns), ost::Error);
     columns.pop_back();
   }
   BOOST_TEST_MESSAGE("          done.");
@@ -1424,4 +1417,108 @@ BOOST_AUTO_TEST_CASE(mmcif_test_revisions_new)
   BOOST_TEST_MESSAGE("  done.");
 }
 
+BOOST_AUTO_TEST_CASE(mmcif_pdbx_entity_branch_tests)
+{
+  BOOST_TEST_MESSAGE("  Running mmcif_pdbx_entity_branch_tests...");
+  StarLoopDesc tmmcif_h;
+  mol::EntityHandle eh = mol::CreateEntity();
+
+  BOOST_TEST_MESSAGE("          testing chain type recognition...");
+  {
+    TestMMCifReaderProtected tmmcif_p("testfiles/mmcif/atom_site.mmcif", eh);
+    std::vector<StringRef> columns;
+
+    // create corresponding entity entry
+    tmmcif_h.Clear();
+    tmmcif_h.SetCategory(StringRef("entity", 6));
+    tmmcif_h.Add(StringRef("id", 2));
+    tmmcif_h.Add(StringRef("type", 4));
+    tmmcif_p.OnBeginLoop(tmmcif_h);
+    columns.push_back(StringRef("1", 1));
+    columns.push_back(StringRef("branched", 8));
+    tmmcif_p.ParseEntity(columns);
+    columns.pop_back();
+    columns.pop_back();
+
+    // build dummy pdbx_entity_branch header
+    tmmcif_h.Clear();
+    tmmcif_h.SetCategory(StringRef("pdbx_entity_branch", 18));
+    tmmcif_h.Add(StringRef("entity_id", 9));
+    tmmcif_h.Add(StringRef("type", 4));
+    tmmcif_p.OnBeginLoop(tmmcif_h);
+    columns.push_back(StringRef("1", 1));
+    columns.push_back(StringRef("oligosaccharide", 15));
+    BOOST_CHECK_NO_THROW(tmmcif_p.ParsePdbxEntityBranch(columns));
+    columns.pop_back();
+
+    columns.push_back(StringRef("ordinarysugar", 13));
+    BOOST_CHECK_THROW(tmmcif_p.ParsePdbxEntityBranch(columns), ost::Error);
+  }
+  BOOST_TEST_MESSAGE("          done.");
+  BOOST_TEST_MESSAGE("  done.");
+}
+
+BOOST_AUTO_TEST_CASE(mmcif_pdbx_entity_branch_link_tests)
+{
+  BOOST_TEST_MESSAGE("  Running mmcif_pdbx_entity_branch_link_tests...");
+  mol::EntityHandle eh = mol::CreateEntity();
+  StarLoopDesc tmmcif_h;
+
+  BOOST_TEST_MESSAGE("          testing link retrieval...");
+  {
+    TestMMCifReaderProtected tmmcif_p("testfiles/mmcif/atom_site.mmcif", eh);
+    std::vector<StringRef> columns;
+
+    // build dummy pdbx_entity_branch_link header
+    tmmcif_h.Clear();
+    tmmcif_h.SetCategory(StringRef("pdbx_entity_branch_link", 23));
+    tmmcif_h.Add(StringRef("entity_id", 9));
+    tmmcif_h.Add(StringRef("atom_id_1", 9));
+    tmmcif_h.Add(StringRef("atom_id_2", 9));
+    tmmcif_h.Add(StringRef("comp_id_1", 9));
+    tmmcif_h.Add(StringRef("comp_id_2", 9));
+    tmmcif_h.Add(StringRef("entity_branch_list_num_1", 24));
+    tmmcif_h.Add(StringRef("entity_branch_list_num_2", 24));
+    tmmcif_p.OnBeginLoop(tmmcif_h);
+    columns.push_back(StringRef("1", 1));
+    columns.push_back(StringRef("C1", 2));
+    columns.push_back(StringRef("O3", 2));
+    columns.push_back(StringRef("MAN", 3));
+    columns.push_back(StringRef("BMA", 3));
+    columns.push_back(StringRef("2", 1));
+    columns.push_back(StringRef("1", 1));
+    BOOST_CHECK_NO_THROW(tmmcif_p.ParsePdbxEntityBranchLink(columns));
+  }
+  BOOST_TEST_MESSAGE("          done.");
+  BOOST_TEST_MESSAGE("          testing link with bond order...");
+  {
+    TestMMCifReaderProtected tmmcif_p("testfiles/mmcif/atom_site.mmcif", eh);
+    std::vector<StringRef> columns;
+
+    // build dummy pdbx_entity_branch_link header
+    tmmcif_h.Clear();
+    tmmcif_h.SetCategory(StringRef("pdbx_entity_branch_link", 23));
+    tmmcif_h.Add(StringRef("entity_id", 9));
+    tmmcif_h.Add(StringRef("atom_id_1", 9));
+    tmmcif_h.Add(StringRef("atom_id_2", 9));
+    tmmcif_h.Add(StringRef("comp_id_1", 9));
+    tmmcif_h.Add(StringRef("comp_id_2", 9));
+    tmmcif_h.Add(StringRef("entity_branch_list_num_1", 24));
+    tmmcif_h.Add(StringRef("entity_branch_list_num_2", 24));
+    tmmcif_h.Add(StringRef("value_order", 11));
+    tmmcif_p.OnBeginLoop(tmmcif_h);
+    columns.push_back(StringRef("1", 1));
+    columns.push_back(StringRef("C1", 2));
+    columns.push_back(StringRef("O3", 2));
+    columns.push_back(StringRef("MAN", 3));
+    columns.push_back(StringRef("BMA", 3));
+    columns.push_back(StringRef("2", 1));
+    columns.push_back(StringRef("1", 1));
+    columns.push_back(StringRef("2", 1));
+    BOOST_CHECK_NO_THROW(tmmcif_p.ParsePdbxEntityBranchLink(columns));
+  }
+  BOOST_TEST_MESSAGE("          done.");
+  BOOST_TEST_MESSAGE("  done.");
+}
+
 BOOST_AUTO_TEST_SUITE_END();
